package org.zeroturnaround.javarebel.integration.glassfish;

import org.zeroturnaround.bundled.javassist.CannotCompileException;
import org.zeroturnaround.bundled.javassist.ClassPool;
import org.zeroturnaround.bundled.javassist.CtClass;
import org.zeroturnaround.bundled.javassist.CtNewMethod;
import org.zeroturnaround.bundled.javassist.NotFoundException;
import org.zeroturnaround.javarebel.integration.support.JavassistClassBytecodeProcessor;

/**
 * Transforms <code>org.apache.felix.framework.searchpolicy.ModuleImpl</code>.
 * 
 * @author Rein Raudjärv
 * 
 * @see FelixModuleClassLoaderCBP
 */
public class FelixModuleImplCBP extends JavassistClassBytecodeProcessor {

  private static final String REBEL_ORG_PACKAGE = "org.zeroturnaround";
  private static final String REBEL_COM_PACKAGE = "com.zeroturnaround";

  public void process(ClassPool cp, ClassLoader cl, CtClass ctClass) throws Exception {
    cp.importPackage("org.zeroturnaround.javarebel");

    processBootDelegation(ctClass);

    addClassResourceSource(cp, ctClass);

    processResourceFinders(ctClass);
  }

  /**
   * Enables to see JRebel classes loaded by the system class loader.
   * 
   * By default class loading is not delegated to the parent class loader. 
   */
  private void processBootDelegation(CtClass ctClass) throws CannotCompileException, NotFoundException {
    ctClass.getDeclaredMethod("shouldBootDelegate").insertBefore(
        "{" +
        "  if ($1.startsWith(\"" + REBEL_ORG_PACKAGE + "\")" +
        "   || $1.startsWith(\"" + REBEL_COM_PACKAGE + "\"))" +
        "    return true;" +
    "}");
  }

  /**
   * Enables JRebel to find resources using this module.
   * 
   * Methods <code>getResourceLocal</code> and <code>getResourcesLocal</code> are invoked.
   */
  private void addClassResourceSource(ClassPool cp, CtClass ctClass) throws NotFoundException, CannotCompileException {
    cp.importPackage("java.net");
    cp.importPackage("java.util");
    cp.importPackage("org.zeroturnaround.javarebel.integration.util");

    ctClass.addInterface(cp.get("org.zeroturnaround.javarebel.ClassResourceSource"));

    ctClass.addMethod(CtNewMethod.make(
        "public Resource getClassResource(String className) {" +
        "  return ResourceUtil.getClassResource((ResourceSource) this, className);" +
        "}",
        ctClass));

    ctClass.addMethod(CtNewMethod.make(
        "public Resource getLocalResource(String name) {" +
        "  URL url = getResourceLocal(name);" +

        "  if (url == null)" +
        "    return null;" +

        "  return ResourceUtil.asResource(url);" +
        "}",
        ctClass));

    ctClass.addMethod(CtNewMethod.make(
        "public Resource[] getLocalResources(String name) {" +
        "  Enumeration en = getResourcesLocal(name);" +
        "  if (en == null)" +
        "    return null;" +
        "" +
        "  URL[] urls = ResourceUtil.toURLs(en);" +
        "  if (urls == null)" +
        "    return null;" +

        "  Resource[] result = new Resource[urls.length];" +
        "  for (int i = 0; i < urls.length; i++)" +
        "    result[i] = ResourceUtil.asResource(urls[i]);" +
        "  return result;" +
        "}",
        ctClass));
  }

  /**
   * Delegates resource finding to JRebel.
   */
  private void processResourceFinders(CtClass ctClass) throws CannotCompileException, NotFoundException {
    ctClass.getDeclaredMethod("getResourceLocal").insertBefore(
        "{" + 
        "  Integration integration = IntegrationFactory.getInstance();" +
        "  if (integration.isResourceReplaced(m_classLoader, $1))" +
        "    return integration.findResource(m_classLoader, $1);" + 
    "}");

    ctClass.getDeclaredMethod("getResourcesLocal").insertBefore(
        "{" + 
        "  Integration integration = IntegrationFactory.getInstance();" +
        "  if (integration.isResourceReplaced(m_classLoader, $1))" +
        "    return integration.findResources(m_classLoader, $1);" + 
    "}");
  }

}
